==> sagua_0.2.0-1.tar.gz: sagua-0.2.0/include/scopeguard.h <==
#ifndef GUARD_scopeguard_h
#define GUARD_scopeguard_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This code from http://www.cuj.com/documents/s=8000/cujcexp1812alexandr/alexandr.htm
// 
// by Andrei.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file scopeguard.h
/// @brief ScopeGuard class for exception safe.

/// @brief ScopeGuard base implmentation.
class ScopeGuardImplBase
{
    public:
	void Dismiss() const throw()
	{
	    dismissed_ = true;
	}

    protected:
	ScopeGuardImplBase()
	: dismissed_(false) {}

	ScopeGuardImplBase(const ScopeGuardImplBase& other)
	: dismissed_(other.dismissed_)
	{
	    other.Dismiss();
	}

	~ScopeGuardImplBase() {}

	mutable bool dismissed_;

    private:
	ScopeGuardImplBase& operator=(const ScopeGuardImplBase&);
};

/// @brief Zero parameter ScopeGuard.
template <typename Fun>
class ScopeGuardImpl0 : public ScopeGuardImplBase
{
    public:
	ScopeGuardImpl0(const Fun& fun)
	: fun_(fun)
	{}

	~ScopeGuardImpl0()
	{
	    if(!dismissed_)
		fun_();
	}

    private:
	Fun fun_;
};

/// @brief Zero parameter ScopeGuard helper.
template <typename Fun>
ScopeGuardImpl0<Fun>
MakeGuard(const Fun fun)
{
    return ScopeGuardImpl0<Fun>(fun);
}

/// @brief One paramter ScopeGuard.
template <typename Fun, typename Parm>
class ScopeGuardImpl1 : public ScopeGuardImplBase
{
    public:
	ScopeGuardImpl1(const Fun& fun, const Parm& parm)
	: fun_(fun), parm_(parm)
	{}

	~ScopeGuardImpl1()
	{
	    if(!dismissed_)
		fun_(parm_);
	}

    private:
	Fun fun_;
	const Parm parm_;
};

/// @brief One parameter ScopeGuard helper.
template <typename Fun, typename Parm>
ScopeGuardImpl1<Fun, Parm>
MakeGuard(const Fun fun, const Parm& parm)
{
    return ScopeGuardImpl1<Fun, Parm>(fun, parm);
}

/// @brief One paramter ScopeGuard.
template <typename Fun, typename Parm1, typename Parm2>
class ScopeGuardImpl2 : public ScopeGuardImplBase
{
    public:
	ScopeGuardImpl2(const Fun& fun, const Parm1& parm1, const Parm2& parm2)
	: fun_(fun), parm1_(parm1), parm2_(parm2)
	{}

	~ScopeGuardImpl2()
	{
	    if(!dismissed_)
		fun_(parm1_, parm2_);
	}

    private:
	Fun fun_;
	const Parm1 parm1_;
	const Parm2 parm2_;
};

/// @brief One parameter ScopeGuard helper.
template <typename Fun, typename Parm1, typename Parm2>
ScopeGuardImpl2<Fun, Parm1, Parm2>
MakeGuard(const Fun fun, const Parm1& parm1, const Parm2& parm2)
{
    return ScopeGuardImpl2<Fun, Parm1, Parm2>(fun, parm1, parm2);
}

/// @brief Zero parameter ScopeGuard for member function.
template <class Obj, typename MemFun>
class ObjScopeGuardImpl0 : public ScopeGuardImplBase
{
    public:
	ObjScopeGuardImpl0(Obj& obj, MemFun memFun)
	: obj_(obj), memFun_(memFun)
	{}

	~ObjScopeGuardImpl0()
	{
	    if(!dismissed_) {
		try {
		    
		    (obj_.*memFun_)();
		    
		} catch(...) {
		    

		}
	    }
	}
	
    private:
	Obj obj_;
	MemFun memFun_;
};

/// @brief Zero parameter ScopeGuard for member function helper.
template <class Obj, typename MemFun>
ObjScopeGuardImpl0<Obj, MemFun>
MakeObjGuard(Obj obj, MemFun memFun)
{
    return ObjScopeGuardImpl0<Obj, MemFun>(obj, memFun);
}

/// @brief Class for supporting parameters by reference.
template <class T>
class RefHolder
{
    public:
	RefHolder(T& ref)
	: ref_(ref)
	{}

	operator T& () const
	{
	    return ref_;
	}

    private:
	T& ref_;	
};

/// @brief Class for supporting parameters by reference helper.
template <class T>
inline RefHolder<T>
ByRef(T& t)
{
    return RefHolder<T>(t);
}

typedef const ScopeGuardImplBase& ScopeGuard;

#endif

==> sagua_0.2.5-1.tar.gz: sagua-0.2.5/include/scopeguard.h <==
#ifndef GUARD_scopeguard_h
#define GUARD_scopeguard_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This code from http://www.cuj.com/documents/s=8000/cujcexp1812alexandr/alexandr.htm
// 
// by Andrei.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file scopeguard.h
/// @brief ScopeGuard class for exception safe.

/// @brief ScopeGuard base implmentation.
class ScopeGuardImplBase
{
    public:
	void Dismiss() const throw()
	{
	    dismissed_ = true;
	}

    protected:
	ScopeGuardImplBase()
	: dismissed_(false) {}

	ScopeGuardImplBase(const ScopeGuardImplBase& other)
	: dismissed_(other.dismissed_)
	{
	    other.Dismiss();
	}

	~ScopeGuardImplBase() {}

	mutable bool dismissed_;

    private:
	ScopeGuardImplBase& operator=(const ScopeGuardImplBase&);
};

/// @brief Zero parameter ScopeGuard.
template <typename Fun>
class ScopeGuardImpl0 : public ScopeGuardImplBase
{
    public:
	ScopeGuardImpl0(const Fun& fun)
	: fun_(fun)
	{}

	~ScopeGuardImpl0()
	{
	    if(!dismissed_)
		fun_();
	}

    private:
	Fun fun_;
};

/// @brief Zero parameter ScopeGuard helper.
template <typename Fun>
ScopeGuardImpl0<Fun>
MakeGuard(const Fun fun)
{
    return ScopeGuardImpl0<Fun>(fun);
}

/// @brief One paramter ScopeGuard.
template <typename Fun, typename Parm>
class ScopeGuardImpl1 : public ScopeGuardImplBase
{
    public:
	ScopeGuardImpl1(const Fun& fun, const Parm& parm)
	: fun_(fun), parm_(parm)
	{}

	~ScopeGuardImpl1()
	{
	    if(!dismissed_)
		fun_(parm_);
	}

    private:
	Fun fun_;
	const Parm parm_;
};

/// @brief One parameter ScopeGuard helper.
template <typename Fun, typename Parm>
ScopeGuardImpl1<Fun, Parm>
MakeGuard(const Fun fun, const Parm& parm)
{
    return ScopeGuardImpl1<Fun, Parm>(fun, parm);
}

/// @brief One paramter ScopeGuard.
template <typename Fun, typename Parm1, typename Parm2>
class ScopeGuardImpl2 : public ScopeGuardImplBase
{
    public:
	ScopeGuardImpl2(const Fun& fun, const Parm1& parm1, const Parm2& parm2)
	: fun_(fun), parm1_(parm1), parm2_(parm2)
	{}

	~ScopeGuardImpl2()
	{
	    if(!dismissed_)
		fun_(parm1_, parm2_);
	}

    private:
	Fun fun_;
	const Parm1 parm1_;
	const Parm2 parm2_;
};

/// @brief One parameter ScopeGuard helper.
template <typename Fun, typename Parm1, typename Parm2>
ScopeGuardImpl2<Fun, Parm1, Parm2>
MakeGuard(const Fun fun, const Parm1& parm1, const Parm2& parm2)
{
    return ScopeGuardImpl2<Fun, Parm1, Parm2>(fun, parm1, parm2);
}

/// @brief Zero parameter ScopeGuard for member function.
template <class Obj, typename MemFun>
class ObjScopeGuardImpl0 : public ScopeGuardImplBase
{
    public:
	ObjScopeGuardImpl0(Obj& obj, MemFun memFun)
	: obj_(obj), memFun_(memFun)
	{}

	~ObjScopeGuardImpl0()
	{
	    if(!dismissed_) {
		try {
		    
		    (obj_.*memFun_)();
		    
		} catch(...) {
		    

		}
	    }
	}
	
    private:
	Obj& obj_;
	MemFun memFun_;
};

/// @brief Zero parameter ScopeGuard for member function helper.
template <class Obj, typename MemFun>
ObjScopeGuardImpl0<Obj, MemFun>
MakeObjGuard(Obj& obj, MemFun memFun)
{
    return ObjScopeGuardImpl0<Obj, MemFun>(obj, memFun);
}

/// @brief Class for supporting parameters by reference.
template <class T>
class RefHolder
{
    public:
	RefHolder(T& ref)
	: ref_(ref)
	{}

	operator T& () const
	{
	    return ref_;
	}

    private:
	T& ref_;	
};

/// @brief Class for supporting parameters by reference helper.
template <class T>
inline RefHolder<T>
ByRef(T& t)
{
    return RefHolder<T>(t);
}

typedef const ScopeGuardImplBase& ScopeGuard;

#endif
